@@ -27,10 +27,15 @@ attributes :py:attr:`TrainStore.engine`,
 """
 import dataclasses as _python_dataclasses
 import datetime
+import os
 import sqlite3
 import urllib.parse
 from typing import Any, Dict, List, Literal, Optional, Sequence, Union
 
+import alembic.command
+import alembic.config
+import alembic.migration
+import alembic.operations
 import pandas as pd
 import sqlalchemy.dialects.postgresql
 import sqlalchemy.dialects.sqlite
@@ -52,16 +57,20 @@ from sqlalchemy import (
     func,
 )
 from sqlalchemy import insert as default_insert
-from sqlalchemy import select, text
+from sqlalchemy import inspect, select, text
 from sqlalchemy.engine import Connection, Engine
 from sqlalchemy.exc import IntegrityError
 
 from scalarstop._datetime import utcnow
 from scalarstop.exceptions import SQLite_JSON_ModeDisabled
-from scalarstop.hyperparams import enforce_dict
+from scalarstop.hyperparams import enforce_dict, flatten_hyperparams
 
 _LOGGER = Logger(__name__)
 
+_ALEMBIC_SCRIPT_DIR = os.path.join(
+    os.path.dirname(os.path.dirname(__file__)), "alembic"
+)
+
 _TABLE_NAME_PREFIX = "scalarstop__"
 
 _datablob_value_error = ValueError(
@@ -92,6 +101,7 @@ class _ModelMetadata:
     datablob_name: str
     datablob_group_name: str
     datablob_hyperparams: Dict[str, Any]
+    datablob_hyperparams_flat: Dict[str, Any]
     model_template_name: str
     model_template_group_name: str
     model_template_hyperparams: Dict[str, Any]
@@ -142,7 +152,11 @@ def _flatten_hyperparam_results(results) -> List[Dict[str, str]]:
     results_dicts = []
     for row in results:
         row_dict = dict(row)
-        dbh = row_dict.pop("datablob_hyperparams")
+        dbh = row_dict.pop("datablob_hyperparams_flat")
+        if dbh is None:
+            dbh = row_dict.pop("datablob_hyperparams")
+        else:
+            row_dict.pop("datablob_hyperparams")
         mth = row_dict.pop("model_template_hyperparams")
         results_dicts.append(
             dict(
@@ -187,6 +201,7 @@ class _TrainStoreTables:
                 onupdate=utcnow,
                 nullable=False,
             ),
+            Column("datablob_hyperparams_flat", JSON, nullable=True),
             extend_existing=True,
         )
 
@@ -401,8 +416,32 @@ class TrainStore:
         self._table = _TrainStoreTables(
             table_name_prefix=self._table_name_prefix, dialect=self._engine.name
         )
+
+        # Create all of the database columns at once.
         with self._connection.begin():
+            # Create the database tables if they do not exist.
+            # But do not add columns or alter tables that already exist.
             self._table.metadata.create_all(bind=self._engine)
+            # Set up the inspector so we can csee if create_all()
+            # didn't create all of the columns that we wanted.
+            inspector = inspect(self._connection)
+            # Configure Alembic so we can manually add columns
+            # in this database transaction.
+            context = alembic.migration.MigrationContext.configure(self._connection)
+            op = alembic.operations.Operations(context)
+            # Iterate over the ScalarStop database tables,
+            # manually adding any database columns that happen to be missing.
+            for table_name, expected_table in self._table.metadata.tables.items():
+                actual_column_names = {
+                    col["name"] for col in inspector.get_columns(table_name)
+                }
+                for expected_col in expected_table.columns:
+                    if expected_col.name not in actual_column_names:
+                        _LOGGER.warning(
+                            "Manually creating the new database column "
+                            f"`{table_name}.{expected_col.name}`."
+                        )
+                        op.add_column(table_name, expected_col)  # type: ignore
 
     def __repr__(self) -> str:
         return f"<sp.TrainStore {self._connection_string_no_password}>"
@@ -536,6 +575,7 @@ class TrainStore:
             datablob_name=name,
             datablob_group_name=group_name,
             datablob_hyperparams=enforce_dict(hyperparams),
+            datablob_hyperparams_flat=flatten_hyperparams(hyperparams),
         )
         self._insert(
             table=self.table.datablob,
@@ -555,6 +595,9 @@ class TrainStore:
                 self.table.datablob.c.datablob_name.label("name"),
                 self.table.datablob.c.datablob_group_name.label("group_name"),
                 self.table.datablob.c.datablob_hyperparams.label("hyperparams"),
+                self.table.datablob.c.datablob_hyperparams_flat.label(
+                    "hyperparams_flat"
+                ),
                 self.table.datablob.c.datablob_last_modified.label("last_modified"),
             ]
         ).select_from(self.table.datablob)
@@ -868,6 +911,7 @@ class TrainStore:
                     self.table.datablob.c.datablob_name,
                     self.table.datablob.c.datablob_group_name,
                     self.table.datablob.c.datablob_hyperparams,
+                    self.table.datablob.c.datablob_hyperparams_flat,
                     self.table.model_template.c.model_template_name,
                     self.table.model_template.c.model_template_group_name,
                     self.table.model_template.c.model_template_hyperparams,
@@ -938,6 +982,7 @@ class TrainStore:
             self.table.datablob.c.datablob_name,
             self.table.datablob.c.datablob_group_name,
             self.table.datablob.c.datablob_hyperparams,
+            self.table.datablob.c.datablob_hyperparams_flat,
             self.table.model_template.c.model_template_name,
             self.table.model_template.c.model_template_group_name,
             self.table.model_template.c.model_template_hyperparams,
@@ -1190,7 +1235,7 @@ class TrainStore:
             current_epoch = dict(
                 model_epoch_num=model_epoch_num,
                 model_name=model.name,
-                model_epoch_metrics=dict(),
+                model_epoch_metrics={},
             )
             for metric_name, metric_values in model.history.items():
                 try:
@@ -1365,6 +1410,7 @@ class TrainStore:
             * ``datablob_name``
             * ``datablob_group_name``
             * ``datablob_hyperparams``
+            * ``datablob_hyperparams_flat``
             * ``model_template_name``
             * ``model_template_group_name``
             * ``model_template_hyperparams``
